% ------- fitting a curve to exspected value and variance -------

% Used with permission from its author Endre Sandvik
function [a, b] = meanAndStdDev(scatterD,plotOn)

%Function for establishing coefficients a1,a2,a3 in function for ln(expected
%value) and b1,b2,b3 in function for ln(variance).

%Length of Hs and Tp classes found from scatterdiagram
k = length(scatterD(1,:)); %Hs
d = length(scatterD(:,1)); %Tp

%Declearing vectors for expected value and variance
my = zeros(1,d-3);
variance = zeros(1,d-3);
%%
%Estimating variance and expected value from scatterdiagram

%Double loop for estimating empirical expected value (weighted mean)
% for i = 2:d-2%Hs
%     for j = 2:k-2%Tp
%    
%         %Calculating E[ln(Tp|Hs)] in each class of Hs
%         my(i-1) = my(i-1) + scatterD(i,j)*log(scatterD(1,j))/scatterD(i,k-1);
% 
%     end 
% end
% 
% %Double loop for estimating empirical variance (weighted sum)
% for i = 2:d-2%Hs
%     for j = 2:k-2%Tp
%    
%         %Calculating variance of Tp in each class of Hs
%         
%         if isnan(variance(i-1) + scatterD(i,j)*(log(scatterD(1,j)) - my(i-1))^2/(scatterD(i,k-1)-1)) == 1
%             %variance(i-1) = variance(i-2) + 0.5*(variance(i-2)-variance(i-3));
%         else        
%             variance(i-1) = variance(i-1) + scatterD(i,j)*(log(scatterD(1,j)) - my(i-1))^2/(scatterD(i,k-1)-1);
%         end
%     end 
% end
data_vec = [];
h_var = [];
for i = 2:d-2%Hs
    for j = 2:k-2%Tp
   
        if scatterD(i,j) == 0
            
        else
           data_vec = [data_vec (j-1)*ones(1,scatterD(i,j))];
        end

    end
    
    if length(data_vec) >= 1
        param = lognfit(data_vec);
        my(i-1) = param(1);
        variance(i-1) = param(2)^2;
        data_vec = [];
        h_var = [h_var i-1];
    end
end


%%
%Fitting curve to values of expected value and variance using least square
%curve fitting method

%f1 is the type of function to be fitted to the expected values
f1 = @(a,adata) a(1)+a(2)*adata.^a(3);
%Fitting the curve (f1) to the expected values
a = lsqcurvefit(f1,[0.001 0.001 0.001],scatterD(h_var+1,1)',my(h_var));

%f2 is the type of function to be fitted to the variances
f2 = @(b,bdata) b(1)+b(2)*exp(-bdata*b(3));
%Fitting the curve (f2) to the variances
b = lsqcurvefit(f2,[0.001 0.001 0.001],scatterD(h_var+1,1)',variance(h_var),[0 0 0],[3 3 3]);

%%

if plotOn == 1
    red = [0.8500, 0.3250, 0.0980];
    
    %Plot of empirical values from data for E[lnTp|Hs] and fitted curve for 
    set(figure,'name','Curve fitting expected value','numbertitle','off')
    scatter(scatterD(h_var+1,1),my(h_var),'Marker','x','LineWidth',2);
    hold on
    plot(scatterD(h_var+1,1),f1(a,scatterD(h_var+1,1)),'LineStyle','--','Color',red,'LineWidth',1.5)
    xlabel('$H_{s}$','FontSize',18)
    ylabel('$\mu_{ln T_p(H_s)}$','FontSize',18)
    legend('Calculated Values','Fitted curve','Location','northwest')
    yticks(1.5:0.25:2.5)
    grid on
    
    %Plot of empirical ln(variance) vs fitted curve
    set(figure,'name','Curve fitting variance','numbertitle','off')
    scatter(scatterD(h_var+1,1),variance(h_var),'Marker','x','LineWidth',2);
    hold on
    plot(scatterD(h_var+1,1),f2(b,scatterD(h_var+1,1)),'LineStyle','--','Color',red,'LineWidth',1.5)
    xlabel('$H_{s}$','FontSize',18)
    ylabel('$\sigma^2_{ln T_p(H_s)}$','FontSize',18)
    legend('Calculated Values','Fitted Curve','Location','northeast')
    grid on
end

end