function [obs_cov,sim_cov,obs_month_mean,sim_month_mean] = monthly_mean_covariance(metocean,numRep,Ysim)

% Observed values
HsObs = metocean(:,3);
TpObs = metocean(:,4);
% Initilize
obsMonthMeanHs = zeros(1,12); obsMonthMeanTp = zeros(1,12);
obsCov11 = zeros(1,12);obsCov12 = zeros(1,12);obsCov22 = zeros(1,12);
obsMonthCov = zeros(2,2,12);

% Build matrix/vectors holding mean and cov for each month for observed
for ii = 1:12
    obsMonthMeanHs(ii) = mean(HsObs(metocean(:,2) == ii));
    obsMonthMeanTp(ii) = mean(TpObs(metocean(:,2) == ii));
    obsMonthCov(:,:,ii) = cov(HsObs(metocean(:,2) == ii),TpObs(metocean(:,2) == ii));
end

% Transform cov matrix into vectors for each element
for ii = 1:12
    obsCov11(ii) = obsMonthCov(1,1,ii);
    obsCov12(ii) = obsMonthCov(1,2,ii);
    obsCov22(ii) = obsMonthCov(2,2,ii);
end

% Monthly mean and covariances
obs_month_mean = [obsMonthMeanHs;obsMonthMeanTp];
obs_cov = [obsCov11;obsCov12;obsCov22];

% Simulated values
% Initialize
simMonthMeanHsRep = zeros(numRep,12);simMonthMeanTpRep = zeros(numRep,12);
simMonthCovRep = zeros(2,2,12);
simCov11 = zeros(1,12);simCov12 = zeros(1,12);simCov22 = zeros(1,12);

% Build matrix/vectors holding mean and cov for each month for replications
for ii = 1:numRep
    for jj = 1:12
        simMonthMeanHsRep(ii,jj) = mean(Ysim(metocean(:,2) == jj,1,ii));
        simMonthMeanTpRep(ii,jj) = mean(Ysim(metocean(:,2) == jj,2,ii));
        % Adding cov matrices, then dividing due to matrix structure
        simMonthCovRep(:,:,jj) = simMonthCovRep(:,:,jj)+...
            cov(Ysim(metocean(:,2)==jj,1,ii),Ysim(metocean(:,2)==jj,2,ii));
    end
end


% Monthly mean and cov over all replications
sim_month_mean = [mean(simMonthMeanHsRep);mean(simMonthMeanTpRep)];
simMonthCov = simMonthCovRep/numRep; 

% Transform cov matrix into vectors for each element
for ii = 1:12
    simCov11(ii) = simMonthCov(1,1,ii);
    simCov12(ii) = simMonthCov(1,2,ii);
    simCov22(ii) = simMonthCov(2,2,ii);
end

sim_cov = [simCov11;simCov12;simCov22];
end