function [yearMeanFit,yearCovFit,midMonthHours] = seasonal_trans(startYear,endYear,metoceanBox,year_length)
% Calculate the points necessary to fit Fourier approximations for 
% covariance and mean

% Number of years 
nYears = endYear-startYear+1;
% Number of months 
nMonths = 12*nYears;

% Initilize
monthHsMean = zeros(1,nMonths); monthTpMean = zeros(1,nMonths);
monthCov = zeros(2,2,nMonths);
counter = 1; % Set start month to 1

%% Monthly mean and standard deviation
% Build matrix of mean and standard deviation for each month in the series
for ii = startYear:1:endYear
    for jj = 1:12
        monthHsMean(counter) = mean(metoceanBox(metoceanBox(:,1) == ii & metoceanBox(:,2) == jj,3));
        monthTpMean(counter) = mean(metoceanBox(metoceanBox(:,1) == ii & metoceanBox(:,2) == jj,4));
        monthCov(:,:,counter) = cov(metoceanBox(metoceanBox(:,1) == ii & metoceanBox(:,2) == jj,3), ...
            metoceanBox(metoceanBox(:,1) == ii & metoceanBox(:,2) == jj,4));
        counter = counter + 1;
    end
end

%% Yearly Mean and Standard Deviation
% Build matrix of mean and standard deviation for the average of each month
% over all years in the series

% Initilize
yearHsMeanSum = zeros(1,12); yearTpMeanSum = zeros(1,12);
yearCovSum = zeros(2,2,12);

% Yearly mean
% Sum for each month over all years
for ii = 1:12
    for jj = 1:nYears
        yearHsMeanSum(ii) = yearHsMeanSum(ii) + monthHsMean(ii+12*(jj-1));
        yearTpMeanSum(ii) = yearTpMeanSum(ii) + monthTpMean(ii+12*(jj-1));
        yearCovSum(:,:,ii) = yearCovSum(:,:,ii) + monthCov(:,:,ii+12*(jj-1));
    end
end

% Divide by number of years to find average
yearTpMean = yearTpMeanSum/nYears;
yearHsMean = yearHsMeanSum/nYears; 
yearCov = yearCovSum/nYears;

% Combine mean values
yearMean = transpose([yearHsMean;yearTpMean]);

%% Seasonal Transformation
% Mean periodic function M

% Vector containing the middle day of each month in hours from year start
% Day 1 = 365, such that the period is equal to one year 
midMonthHours = transpose([1 15.5 45 74.5 105 135.5 166 196.5 227.5 258 288.5 319 349.5 365]*(year_length/365));

% Row containing the average mean value for the 1st and 365th day
% Found as the mean of month 1 and month 12
startEndDay(1,:) = (yearMean(1,:) + yearMean(end,:))/2;

% Add the average value row as the first and last row in month mean matrix
yearMeanFit = [startEndDay;yearMean;startEndDay];



% Covariance periodic function SIGMA

% Initialize
S11 = zeros(12,1);S12 = zeros(12,1);S22 = zeros(12,1);

% Build vector for each covariance per month
for ii = 1:12
    S11(ii,1) = yearCov(1,1,ii);
    S12(ii,1) = yearCov(1,2,ii); % S12 = S21
    S22(ii,1) = yearCov(2,2,ii);
end

% Combine monthly values of covariance into monthly covariance matrix
obsMonthCov = [S11 S12 S12 S22];

% Row containing the average mean value for the 1st and 365th day
% Found as the mean of month 1 and month 12
startEndDay2(1,:) = (obsMonthCov(1,:) + obsMonthCov(end,:))/2;

% Add the average value row as the first and last row in month mean matrix
yearCovFit = [startEndDay2;obsMonthCov;startEndDay2];

end