% Spread calm weather observations
function [Tp_spread] = spread_calm_peak_period(metocean)

% Distriubute large first
cut_off_large = 100;

% Count number of occurrences of each unique Tp
unique_Tp = unique(metocean(:,4));
count_bin = zeros(length(unique_Tp),1);
for ii = 1:length(unique_Tp)
    count_bin(ii,1) = sum(metocean(:,4) == unique_Tp(ii));
end

% value of the clusters
cluster_val = unique_Tp(count_bin > cut_off_large);

% Merge close clusters
delta = 0.065; % 0.065
merge_list = zeros(length(cluster_val)-1,1);

% Identify clusters to merge
for ii = 1:length(cluster_val)-1
    if cluster_val(ii+1) - cluster_val(ii) < delta
        merge_list(ii) = 1; 
        merge_list(ii+1) = 2; 
    end
end

% Add zero for last element
merge_list(end+1) = 0;

% Vector holding target value for old clusters
merge_target1 = cluster_val(merge_list==1);
merge_target2 = cluster_val(merge_list==2);

merge_target = zeros(1,length(merge_list));

for ii = 0:(length(merge_target1)-1)
    merge_target(2*ii+1) = merge_target1(ii+1);
    merge_target(2*(ii+1)) = merge_target2(ii+1);
end

merged_new_value = zeros(1,length(merge_target));

% New values for clusters
counter = 1;
for ii = 1:length(merge_list)-1
    if merge_list(ii) == 1
        merged_new_value(counter) = (cluster_val(ii)+cluster_val(ii+1))/2;
        counter = counter + 1;
    elseif merge_list(ii) == 2
        merged_new_value(counter) = (cluster_val(ii)+cluster_val(ii-1))/2;
        counter = counter + 1;
    end
    
end

% Merge clusters 
for ii = 1:length(metocean)
    for kk = 1:length(merge_target)
        if metocean(ii,4) == merge_target(kk)
            metocean(ii,4) = merged_new_value(kk);
            break
        end
    end
end

% Count number of occurrences of each unique Tp
unique_clustered_metocean = unique(metocean(:,4));
count_bin_cluster = zeros(length(unique_clustered_metocean),1);
for ii = 1:length(unique_clustered_metocean)
    count_bin_cluster(ii,1) = sum(metocean(:,4) == unique_clustered_metocean(ii));
end

cluster_val = unique_clustered_metocean(count_bin_cluster > cut_off_large);
cluster_size = count_bin_cluster(count_bin_cluster > cut_off_large);

% Unform distribution of the clusters using number of occurences as weights
for ii = 1:length(metocean(:,4))
    for jj = 2:(length(cluster_val)-1)
        if metocean(ii,4) == cluster_val(1)
            dist = cluster_val(2) - cluster_val(1);
            weight = cluster_val(2)/cluster_val(1);
            if rand < weight
                metocean(ii,4) = metocean(ii,4) + dist*rand;
            else
                metocean(ii,4) = metocean(ii,4) - dist*rand;
            end
            break
        elseif metocean(ii,4) == cluster_val(jj)
            dist_1 = cluster_val(jj)-cluster_val(jj-1);
            dist_2 = cluster_val(jj+1)-cluster_val(jj);
            weight = cluster_size(jj-1)/(cluster_size(jj-1)+cluster_size(jj+1));
            if rand < weight
                metocean(ii,4) = cluster_val(jj) - dist_1*rand;
            else
                metocean(ii,4) = cluster_val(jj) + dist_2*rand;
            end
            break
        elseif metocean(ii,4) == cluster_val(end)
            dist = 2*(cluster_val(end) - cluster_val(end-1));
            metocean(ii,4) = cluster_val(end-1) + dist*rand;
            break
        end
    end
end

% Distibute remaining small clusters
cluster_val = unique_clustered_metocean(count_bin_cluster < cut_off_large & count_bin_cluster > 40);
cluster_size = count_bin_cluster(count_bin_cluster < cut_off_large & count_bin_cluster > 40);


for ii = 1:length(metocean(:,4))
    for jj = 2:(length(cluster_val)-1)
        if metocean(ii,4) == cluster_val(1)
            dist = cluster_val(2) - cluster_val(1);
            weight = cluster_val(2)/cluster_val(1);
            if rand < weight
                metocean(ii,4) = metocean(ii,4) + dist*rand;
            else
                metocean(ii,4) = metocean(ii,4) - dist*rand;
            end
            break
        elseif metocean(ii,4) == cluster_val(jj)
            dist_1 = cluster_val(jj)-cluster_val(jj-1);
            dist_2 = cluster_val(jj+1)-cluster_val(jj);
            weight = cluster_size(jj-1)/(cluster_size(jj-1)+cluster_size(jj+1));
            if rand < weight
                metocean(ii,4) = cluster_val(jj) - dist_1*rand;
            else
                metocean(ii,4) = cluster_val(jj) + dist_2*rand;
            end
            break
        elseif metocean(ii,4) == cluster_val(end)
            dist = 2*(cluster_val(end) - cluster_val(end-1));
            metocean(ii,4) = cluster_val(end-1) + dist*rand;
            break
        end
    end
end

Tp_spread = metocean(:,4);
end


