% Validate best

clear all

%% Set Latex as interpreter and font size
set(0,'defaultTextInterpreter','latex')
set(0, 'defaultAxesTickLabelInterpreter','latex'); 
set(0, 'defaultLegendInterpreter','latex');

set(0,'DefaultAxesFontSize',16) % 12 for 0.8, 16 for 0.65
set(0,'DefaultTextFontSize',16)
set(0,'DefaultLegendFontSize',16)

% Colors
red = [0.8500, 0.3250, 0.0980];
orangie = [0.9290, 0.6940, 0.1250];
blue = [0, 0.4470, 0.7410];

% General parameters
harsh = 1; % 0 for calm, 1 for harsh
num_rep = 10;
log_scale = 0; % log scale for COV plot

% plot parameters
s1_name = 'VARMA(3,1)$_{f,box}$';
s2_name = 'VARMA(2,3)$_f$'; %'VARMA(2,3)$_f$' 

line_width = 1.5;
num_lags_auto = 100; % Number of lags in auto/crosscorrelation
plot_pers = 0;

% Files
observed_filename = 'WAM10.txt'; % WAM 10 Metocean file without headers. 
series_1_name = 'VARMA_13_BOXCOX_MS_H.csv';
series_2_name = 'VARMA_23_MS_H.csv';

% LOAD PERSISTENCE TABLES
if plot_pers == 1
    obs_persistence = xlsread('pers_table_harsh.xlsx');
    obs_above = obs_persistence(2:61,1:21);
    obs_below = obs_persistence(63:122,1:21);
    syn_persistence_1 = xlsread('pers_table_box.xlsx');
    syn_persistence_2 = xlsread('pers_table_VARMA_23_MS_H.xlsx');
    syn_above_1 = syn_persistence_1(2:61,1:21);
    syn_below_1 = syn_persistence_1(63:122,1:21);
    syn_above_2 = syn_persistence_2(2:61,1:21);
    syn_below_2 = syn_persistence_2(63:122,1:21);
end

% Read observed series
if harsh == 1
    step_length = 3;
    start_year = 1990; % Start year of time series
    end_year = 2015; % Final year of time series, including this year
    year_length = 2920; % 1460 Number of observations in a year
    metocean = loadTimeSeries(start_year,end_year,observed_filename);
    above_values = [.01 .01 .05 .2 .4 .6 .6]; % [.001 .01 .1 .2 .4 .6 .7];
    below_values = [.01 .05 .2 .4 .6 .8 .8]; %[.01 .1 .2 .4 .6 .7 .8];
else
    metocean = csvread(observed_filename);
    step_length = 6;
    year_length = 1460;
    above_values = [.001 .01 .1 .1 .2 .6 .7]; % [.001 .01 .05 .1 .2 .6 .7];
    below_values = [.15 .3 .5 .7 .8 .9 .9];
end



%% Read 
% Read synthetic series
syn_series_1 = csvread(series_1_name);
syn_series_2 = csvread(series_2_name);

% Remove month and year columns
syn_series_1 = syn_series_1(:,3:end);
syn_series_2 = syn_series_2(:,3:end);

% Reshape synthetic series
Y_sim_1 = zeros(length(syn_series_1),2,num_rep);
Y_sim_2 = zeros(length(syn_series_2),2,num_rep);
for ii = 1:num_rep
    Y_sim_1(:,1,ii) = syn_series_1(:,2*(ii-1)+1);
    Y_sim_1(:,2,ii) = syn_series_1(:,2*ii);
    Y_sim_2(:,1,ii) = syn_series_2(:,2*(ii-1)+1);
    Y_sim_2(:,2,ii) = syn_series_2(:,2*ii);
end

%% Uniformly distribute Tp
if harsh == 1
    % Spread Tp with uniform distribution
    for ii = 1:length(metocean(:,4))
        n = round(1+log(metocean(ii,4)/3.244)/0.09525);
        metocean(ii,4) = 3.244*exp(0.09525*(n-0.5-rand));
    end
else 
    metocean(:,4) = spread_calm_peak_period(metocean);
end

Hs_obs = metocean(:,3);
Tp_obs = metocean(:,4);

%% Overall and Monthly Mean and Covaraince
% Over-all mean and covariance
overall_mean_cov_1 = over_all_mean_variance(metocean(:,3),metocean(:,4),Y_sim_1,num_rep);
overall_mean_cov_2 = over_all_mean_variance(metocean(:,3),metocean(:,4),Y_sim_2,num_rep);

overall_mean_cov = [overall_mean_cov_1(1,:); overall_mean_cov_2];

% Monthly Mean and Covariance
[obs_cov,sim_cov_1,obs_month_mean,sim_month_mean_1] = monthly_mean_covariance(metocean,num_rep,Y_sim_1);
[obs_cov,sim_cov_2,obs_month_mean,sim_month_mean_2] = monthly_mean_covariance(metocean,num_rep,Y_sim_2);

% Plot monthly mean and covariance
figure(1)
clf
subplot(2,1,1)
plot(1:12,obs_month_mean(1,:),'LineWidth',line_width) % Obs
hold on
plot(1:12,sim_month_mean_2(1,:),'LineWidth',line_width,'LineStyle','--') % Other
plot(1:12,sim_month_mean_1(1,:),'LineWidth',line_width,'LineStyle',':') % LSTM

xlim([1,12])
xticks([1 3 6 9 12])
yticks([0 2 4 6])
ylim([0,6])
ylabel('Mean $H_s$','Color','k')
legend('Observed',s2_name,s1_name)
grid on
subplot(2,1,2)
plot(1:12,obs_month_mean(2,:),'LineWidth',line_width)  % Obs
hold on
plot(1:12,sim_month_mean_2(2,:),'LineWidth',line_width,'LineStyle','--') % Other
plot(1:12,sim_month_mean_1(2,:),'LineWidth',line_width,'LineStyle',':') % LSTM
xlim([1,12])
xlabel('Month','Color','k')
ylabel('Mean $T_p$','Color','k')
xticks([1 3 6 9 12])
ylim([4,10])
yticks([4 6 8 10])
grid on

if log_scale == 1
% Covariance
    figure(2)
    clf
    subplot(3,1,1)
    semilogy(1:12,obs_cov(1,:),'LineWidth',line_width)
    hold on
    semilogy(1:12,sim_cov_2(1,:),'LineWidth',line_width,'LineStyle','--')
    semilogy(1:12,sim_cov_1(1,:),'LineWidth',line_width,'LineStyle',':')
    legend('Observed',s2_name,s1_name)
    ylabel('$\Sigma_{11}$','Color','k')
    xlim([1,12])
    xticks([1 3 6 9 12])
    ylim([0,100])
    yticks([10 100])
    yticklabels({'$10^1$','$10^2$'})
    grid on

    subplot(3,1,2)
    semilogy(1:12,obs_cov(2,:),'LineWidth',line_width)
    hold on
    semilogy(1:12,sim_cov_2(2,:),'LineWidth',line_width,'LineStyle','--')
    semilogy(1:12,sim_cov_1(2,:),'LineWidth',line_width,'LineStyle',':')
    ylabel('$\Sigma_{12}$','Color','k')
    xlim([1,12])
    ylim([0,100])
    xticks([1 3 6 9 12])
    yticks([10 100])
    yticklabels({'$10^1$','$10^2$'})
    grid on

    subplot(3,1,3)
    semilogy(1:12,obs_cov(3,:),'LineWidth',line_width)
    hold on
    semilogy(1:12,sim_cov_2(3,:),'LineWidth',line_width,'LineStyle','--')
    semilogy(1:12,sim_cov_1(3,:),'LineWidth',line_width,'LineStyle',':')
    xlabel('Month','Color','k')
    ylabel('$\Sigma_{22}$','Color','k')
    xlim([1,12])
    ylim([0,100])
    xticks([1 3 6 9 12])
    yticks([10 100])
    yticklabels({'$10^1$','$10^2$'})
    grid on

else
    figure(2)
    clf
    subplot(3,1,1)
    plot(1:12,obs_cov(1,:),'LineWidth',line_width)
    hold on
    plot(1:12,sim_cov_2(1,:),'LineWidth',line_width,'LineStyle','--')
    plot(1:12,sim_cov_1(1,:),'LineWidth',line_width,'LineStyle',':')
    legend('Observed',s2_name,s1_name)
    ylabel('$\Sigma_{11}$','Color','k')
    xlim([1,12])
    xticks([1 3 6 9 12])
    %ylim([0,100])
    %yticks([10 100])
    %yticklabels({'$10^1$','$10^2$'})
    grid on

    subplot(3,1,2)
    plot(1:12,obs_cov(2,:),'LineWidth',line_width)
    hold on
    plot(1:12,sim_cov_2(2,:),'LineWidth',line_width,'LineStyle','--')
    plot(1:12,sim_cov_1(2,:),'LineWidth',line_width,'LineStyle',':')
    ylabel('$\Sigma_{12}$','Color','k')
    xlim([1,12])
    %ylim([0,100])
    xticks([1 3 6 9 12])
    %yticks([10 100])
    %yticklabels({'$10^1$','$10^2$'})
    grid on

    subplot(3,1,3)
    plot(1:12,obs_cov(3,:),'LineWidth',line_width)
    hold on
    plot(1:12,sim_cov_2(3,:),'LineWidth',line_width,'LineStyle','--')
    plot(1:12,sim_cov_1(3,:),'LineWidth',line_width,'LineStyle',':')
    xlabel('Month','Color','k')
    ylabel('$\Sigma_{22}$','Color','k')
    xlim([1,12])
    %ylim([0,100])
    xticks([1 3 6 9 12])
    %yticks([10 100])
    %yticklabels({'$10^1$','$10^2$'})
    grid on
end


%% Autocorrelation and Crosscorrelation

% Hs Autocorrelation
[acf_Hs_sim_1,lags_auto] = autocorr(Y_sim_1(:,1,1),num_lags_auto);
[acf_Hs_sim_2,lags_auto] = autocorr(Y_sim_2(:,1,1),num_lags_auto);
acf_Hs_obs = autocorr(metocean(:,3),num_lags_auto);

% Tp Autocorrelation
acf_Tp_sim_1 = autocorr(Y_sim_1(:,2,1),num_lags_auto);
acf_Tp_sim_2 = autocorr(Y_sim_2(:,2,1),num_lags_auto);
acf_Tp_obs = autocorr(metocean(:,4),num_lags_auto);

figure(3)
clf
subplot(2,1,1)
plot(lags_auto,acf_Hs_obs,'LineWidth',line_width)
hold on
plot(lags_auto,acf_Hs_sim_2,'LineWidth',line_width,'LineStyle','--')
plot(lags_auto,acf_Hs_sim_1,'LineWidth',line_width,'LineStyle',':')
legend('Observed',s2_name,s1_name)
ylabel('ACF $H_s$','Color','k')
grid on

subplot(2,1,2)
plot(lags_auto,acf_Tp_obs,'LineWidth',line_width)
hold on
plot(lags_auto,acf_Tp_sim_2,'LineWidth',line_width,'LineStyle','--')
plot(lags_auto,acf_Tp_sim_1,'LineWidth',line_width,'LineStyle',':')
xlabel('Lag','Color','k')
ylabel('ACF $T_p$','Color','k')
grid on

% Cross Correlation
[xcf_sim_1,lags_cross] = crosscorr(Y_sim_1(:,1,1),Y_sim_1(:,2,1));
[xcf_sim_2,lags_cross] = crosscorr(Y_sim_2(:,1,1),Y_sim_2(:,2,1));
xcf_obs = crosscorr(metocean(:,3),metocean(:,4));

figure(4)
clf
plot(lags_cross,xcf_obs,'LineWidth',line_width)
hold on
plot(lags_cross,xcf_sim_2,'LineWidth',line_width,'LineStyle','--')
plot(lags_cross,xcf_sim_1,'LineWidth',line_width,'LineStyle',':')

xlabel('Lag','Color','k')
ylabel('Crosscorrelation Function','Color','k')
legend('Observed',s2_name,s1_name)
grid on

%% Scatter plot
series_length = length(metocean);

% Preallocation
group_obs = cell(length(syn_series_1),1);
group_1 = cell(length(syn_series_1),1);
group_2 = cell(length(syn_series_1),1);

% Create one group for model and observed
for ii = 1:series_length
    group_obs{ii} = 'Observed';
    group_1{ii} = s1_name;
    group_2{ii} = s2_name;
end

% Vectors for scatterhist
type_1 = cat(1,group_obs,group_1);

HsPlot_1 = [metocean(:,3); Y_sim_1(:,1,1)];
TpPlot_1 = [metocean(:,4); Y_sim_1(:,2,1)];

type_2 = cat(1,group_obs,group_2);

HsPlot_2 = [metocean(:,3); Y_sim_2(:,1,1)];
TpPlot_2 = [metocean(:,4); Y_sim_2(:,2,1)];

%[0 2 4 6 8 10 12 14 16 18 20 22]
figure(5)
clf
scatterhist(TpPlot_1, HsPlot_1,'Group',type_1, 'MarkerSize', 2,'kernel','on',...
    'Color',[blue;orangie],'LineWidth',[line_width,line_width],'Location','SouthWest')
xlim([0 22]) % scatter_max_x
ylim([0 20])
xticks(0:4:22) % scatter_max_x
%xticklabels({'2','4','6','8','10','12','14','16','18','20','22',})
yticks(0:4:20)
ylabel('$H_s$','Color','k')
xlabel('$T_p$','Color','k')
grid on

figure(6)
clf
scatterhist(TpPlot_2, HsPlot_2,'Group',type_2, 'MarkerSize', 2,'kernel','on',...
    'Color',[blue;red],'LineWidth',[line_width,line_width],'Location','SouthWest')
xlim([0 22]) % scatter_max_x
ylim([0 20])
xticks(0:4:22) % scatter_max_x
%xticklabels({'2','4','6','8','10','12','14','16','18','20','22',})
yticks(0:4:20)
ylabel('$H_s$','Color','k')
xlabel('$T_p$','Color','k')
grid on


%% Normalized Persistence Tables

if plot_pers == 1
    
        % Total simulated time
    total_time = length(syn_series_1) * step_length; 

    % Size of persistence tables
    [nRow, nCol] = size(obs_above);

    % Initilize
    hindAboveTime = zeros(nRow,nCol); hindBelowTime = zeros(nRow,nCol);
    simAboveTime_1 = zeros(nRow,nCol); simBelowTime_1 = zeros(nRow,nCol);
    simAboveTime_2 = zeros(nRow,nCol); simBelowTime_2 = zeros(nRow,nCol);
    % Normalize and transform to real time
    for ii = 1:nRow
        hindAboveTime(ii,:) = 6*ii*obs_above(ii,:)/total_time;
        hindBelowTime(ii,:) = 6*ii*obs_below(ii,:)/total_time;
        simAboveTime_1(ii,:) = 6*ii*syn_above_1(ii,:)/total_time;
        simBelowTime_1(ii,:) = 6*ii*syn_below_1(ii,:)/total_time;
        simAboveTime_2(ii,:) = 6*ii*syn_above_2(ii,:)/total_time;
        simBelowTime_2(ii,:) = 6*ii*syn_below_2(ii,:)/total_time;
    end
    
    % Low Values Normalized Persistence ABOVE
    figure(7)
    clf
    v = [above_values(1),above_values(1)];
    [C,h] = contour(flipud(hindAboveTime),v,'-','color',blue,'LineWidth',line_width);
    clabel(C,h)
    hold on
    [C,h] = contour(flipud(simAboveTime_2),v,'--','color',red,'LineWidth',line_width);
    clabel(C,h)
    [C,h] = contour(flipud(simAboveTime_1),v,':','color',orangie,'LineWidth',line_width);
    clabel(C,h)
    
    v = [above_values(2),above_values(2)];
    [C,h] = contour(flipud(hindAboveTime),v,'-','color',blue,'LineWidth',line_width);
    clabel(C,h)
    [C,h] = contour(flipud(simAboveTime_2),v,'--','color',red,'LineWidth',line_width);
    clabel(C,h)
    [C,h] = contour(flipud(simAboveTime_1),v,':','color',orangie,'LineWidth',line_width);
    clabel(C,h)

    v = [above_values(3),above_values(3)];
    [C,h] = contour(flipud(hindAboveTime),v,'-','color',blue,'LineWidth',line_width);
    clabel(C,h)
    [C,h] = contour(flipud(simAboveTime_2),v,'--','color',red,'LineWidth',line_width);
    clabel(C,h)
    [C,h] = contour(flipud(simAboveTime_1),v,':','color',orangie,'LineWidth',line_width);
    clabel(C,h)

    v = [above_values(4),above_values(4)];
    [C,h] = contour(flipud(hindAboveTime),v,'-','color',blue,'LineWidth',line_width);
    clabel(C,h)
    [C,h] = contour(flipud(simAboveTime_2),v,'--','color',red,'LineWidth',line_width);
    clabel(C,h)
    [C,h] = contour(flipud(simAboveTime_1),v,':','color',orangie,'LineWidth',line_width);
    clabel(C,h)

    v = [above_values(5),above_values(5)];
    [C,h] = contour(flipud(hindAboveTime),v,'-','color',blue,'LineWidth',line_width);
    clabel(C,h)
    [C,h] = contour(flipud(simAboveTime_2),v,'--','color',red,'LineWidth',line_width);
    clabel(C,h)
    [C,h] = contour(flipud(simAboveTime_1),v,':','color',orangie,'LineWidth',line_width);
    clabel(C,h)

    v = [above_values(6),above_values(6)];
    [C,h] = contour(flipud(hindAboveTime),v,'-','color',blue,'LineWidth',line_width);
    clabel(C,h)
    [C,h] = contour(flipud(simAboveTime_2),v,'--','color',red,'LineWidth',line_width);
    clabel(C,h)
    [C,h] = contour(flipud(simAboveTime_1),v,':','color',orangie,'LineWidth',line_width);
    clabel(C,h)

    v = [above_values(7),above_values(7)];
    [C,h] = contour(flipud(hindAboveTime),v,'-','color',blue,'LineWidth',line_width);
    clabel(C,h)
    [C,h] = contour(flipud(simAboveTime_2),v,'--','color',red,'LineWidth',line_width);
    clabel(C,h)
    [C,h] = contour(flipud(simAboveTime_1),v,':','color',orangie,'LineWidth',line_width);
    clabel(C,h)
    
    yticks([1 10 20 30 40 50 60])
    yticklabels({'360','300','240','180','120','60','6'})
    xticks([1 5 9 13 17 21])
    xticklabels({'1.0','1.5','2.0','2.5','3.0','3.5'})
    xlabel('Significant Wave Height Threshold [m]','Color','k')
    ylabel('Required Window Length [h]','Color','k')
    lgd = legend('Observed',s2_name,s1_name);
    lgd.Location = 'southeast';
    hold off
    grid on
    
     % Low Values Normalized Persistence Below
    figure(8)
    clf
    v = [below_values(1),below_values(1)];
    [C,h] = contour(flipud(hindBelowTime),v,'-','color',blue,'LineWidth',line_width);
    clabel(C,h)
    hold on
    [C,h] = contour(flipud(simBelowTime_2),v,'--','color',red,'LineWidth',line_width);
    clabel(C,h)
    [C,h] = contour(flipud(simBelowTime_1),v,':','color',orangie,'LineWidth',line_width);
    clabel(C,h)
    
    v = [below_values(2),below_values(2)];
    [C,h] = contour(flipud(hindBelowTime),v,'-','color',blue,'LineWidth',line_width);
    clabel(C,h)
    [C,h] = contour(flipud(simBelowTime_2),v,'--','color',red,'LineWidth',line_width);
    clabel(C,h)
    [C,h] = contour(flipud(simBelowTime_1),v,':','color',orangie,'LineWidth',line_width);
    clabel(C,h)

    v = [below_values(3),below_values(3)];
    [C,h] = contour(flipud(hindBelowTime),v,'-','color',blue,'LineWidth',line_width);
    clabel(C,h)
    [C,h] = contour(flipud(simBelowTime_2),v,'--','color',red,'LineWidth',line_width);
    clabel(C,h)
    [C,h] = contour(flipud(simBelowTime_1),v,':','color',orangie,'LineWidth',line_width);
    clabel(C,h)
    
    v = [below_values(4),below_values(4)];
    [C,h] = contour(flipud(hindBelowTime),v,'-','color',blue,'LineWidth',line_width);
    clabel(C,h)
    [C,h] = contour(flipud(simBelowTime_2),v,'--','color',red,'LineWidth',line_width);
    clabel(C,h)
    [C,h] = contour(flipud(simBelowTime_1),v,':','color',orangie,'LineWidth',line_width);
    clabel(C,h)

    v = [below_values(5),below_values(5)];
    [C,h] = contour(flipud(hindBelowTime),v,'-','color',blue,'LineWidth',line_width);
    clabel(C,h)
    [C,h] = contour(flipud(simBelowTime_2),v,'--','color',red,'LineWidth',line_width);
    clabel(C,h)
    [C,h] = contour(flipud(simBelowTime_1),v,':','color',orangie,'LineWidth',line_width);
    clabel(C,h)

    v = [below_values(6),below_values(6)];
    [C,h] = contour(flipud(hindBelowTime),v,'-','color',blue,'LineWidth',line_width);
    clabel(C,h)
    [C,h] = contour(flipud(simBelowTime_2),v,'--','color',red,'LineWidth',line_width);
    clabel(C,h)
    [C,h] = contour(flipud(simBelowTime_1),v,':','color',orangie,'LineWidth',line_width);
    clabel(C,h)

    v = [below_values(7),below_values(7)];
    [C,h] = contour(flipud(hindBelowTime),v,'-','color',blue,'LineWidth',line_width);
    clabel(C,h)
    [C,h] = contour(flipud(simBelowTime_2),v,'--','color',red,'LineWidth',line_width);
    clabel(C,h)
    [C,h] = contour(flipud(simBelowTime_1),v,':','color',orangie,'LineWidth',line_width);
    clabel(C,h)
    
    yticks([1 10 20 30 40 50 60])
    yticklabels({'360','300','240','180','120','60','6'})
    xticks([1 5 9 13 17 21])
    xticklabels({'1.0','1.5','2.0','2.5','3.0','3.5'})
    xlabel('Significant Wave Height Threshold [m]','Color','k')
    ylabel('Required Window Length [h]','Color','k')
    lgd = legend('Observed',s2_name,s1_name);
    lgd.Location = 'southeast';
    hold off
    grid on
end

