# Simulate residuals using VARMA model

# Set directory
setwd("C:/Users/olejl/OneDrive/Dokumenter/MATLAB/MasterThesis")

# Open time series library
library(MTS)

# Load residuals from Matlab
mydata = read.table("C:/residuals_BOXCOX_MS_H.csv",header=FALSE,sep=",")

# Select Hs and Tp columns
mydata = mydata[,3:4]

# Calculate cross-corrolation P values
P=Eccm(mydata,maxp=4,maxq=4)

# P values matrix
pValues = P$pEccm 

# Find the smallest P value larger than zero
pValueMin = which(pValues == min(pValues[pValues > 0.05]),arr.ind=T)

# Optimal lags by P value
pLags = pValueMin[1]-1 
qLags = pValueMin[2]-1 

# Lag vectors
ARlags = c(1:pLags)
MAlags = c(1:qLags)

# Fit VARMA model to residuals W
mdl=VARMA(mydata,p=pLags,q=qLags,include.mean=FALSE)

# VARMA parameters
AIC = mdl$aic
phi = mdl$Phi
theta = mdl$Theta
sig = mdl$Sigma

series_length = length(mydata)

# Simulate VARMA
m=VARMAsim(series_length,arlags=ARlags,malags=MAlags,
           phi=phi,theta=theta,sigma=sig)

# Simulated residual series
Wsim=m$series

# Write simulation results to .csv
write.table(Wsim,file="Residuals.csv",row.names=FALSE,na="",
            col.names=FALSE,sep=",")